import re
from fastapi import Request, HTTPException, UploadFile
from typing import Dict, Any, Optional, Union, List
from pydantic import BaseModel
import json
from collections import defaultdict

class RequestData(BaseModel):
    queries: Dict[str, Any]
    params: Dict[str, Any]
    headers: Dict[str, Any]
    cookies: Dict[str, Any]
    session: Dict[str, Any]
    body: Optional[Union[Dict[str, Any], str]]
    form_data: Optional[Dict[str, Any]]

async def extract_all_request_data(request: Request) -> RequestData:
    try:
        # Extraction des données de base
        query_params = dict(request.query_params)
        path_params = dict(request.path_params)
        headers = dict(request.headers)
        cookies = dict(request.cookies)
        
        # Gestion du corps et form-data
        body = None
        form_data = None
        content_type = headers.get('content-type', '')

        if 'multipart/form-data' in content_type:
            form_data = await parse_form_data(request)
        elif 'application/x-www-form-urlencoded' in content_type:
            form_data = await parse_urlencoded_form(request)
        elif request.method in ['POST', 'PUT', 'PATCH']:
            try:
                body = await request.json()
            except json.JSONDecodeError:
                body = await request.body()
                try:
                    body = body.decode()
                except:
                    body = str(body)

        # Session data
        session = {}
        if hasattr(request.state, 'session'):
            session = request.state.session
        elif 'session' in request.scope:
            session = request.scope['session']

        return RequestData(
            queries=query_params,
            params=path_params,
            headers=headers,
            cookies=cookies,
            session=session,
            body=body,
            form_data=form_data
        )
    except Exception as e:
        raise HTTPException(
            status_code=500,
            detail=f"Failed to extract request data: {str(e)}"
        )

async def parse_form_data(request: Request) -> Dict[str, Any]:
    form_data = {}
    form_items = await request.form()
    files = await request.files()
    
    # Traitement des champs réguliers
    for key, value in form_items.items():
        if isinstance(value, UploadFile):
            continue  # Géré séparément
        _process_form_field(form_data, key, value)

    # Traitement des fichiers
    for key, file in files.items():
        _process_form_field(form_data, key, file)

    return form_data

async def parse_urlencoded_form(request: Request) -> Dict[str, Any]:
    body = await request.body()
    form_data = {}
    for item in body.decode().split('&'):
        key, value = item.split('=', 1)
        _process_form_field(form_data, key, value)
    return form_data

def _process_form_field(data: dict, key: str, value: Any):
    # Gestion des tableaux (champs comme 'imgs[]')
    if key.endswith('[]'):
        clean_key = key[:-2]
        if clean_key not in data:
            data[clean_key] = []
        data[clean_key].append(value)
        return

    # Gestion des structures imbriquées (other_info[data1])
    if '[' in key and ']' in key:
        parts = re.split(r'\[|\]', key)
        parts = [p for p in parts if p]  # Supprime les chaînes vides
        
        current = data
        for part in parts[:-1]:
            if part not in current:
                current[part] = {}
            current = current[part]
        
        last_part = parts[-1]
        current[last_part] = _convert_value(value)
    else:
        data[key] = _convert_value(value)

def _convert_value(value: Any) -> Any:
    if isinstance(value, UploadFile):
        return value
    
    if isinstance(value, str):
        value = value.strip()
        if value.isdigit():
            return int(value)
        try:
            return float(value)
        except ValueError:
            if value.lower() in ('true', 'false'):
                return value.lower() == 'true'
    return value


from typing import Any, Union, Dict, List, Optional
from fastapi import FastAPI, APIRouter, Request
from pathlib import Path
from .internationalisation_manager import InternationalisationManager
from .config_manager import ConfigManager
from .router import ArcCmsRouter
from .feature_manager import FeatureManager
from .request_utils import RequestData, extract_all_request_data, get_specific_data
import logging
import asyncio

logger = logging.getLogger(__name__)

_intl = InternationalisationManager()
_config = ConfigManager()
_router = ArcCmsRouter()
_feature = FeatureManager()

def init_app(app_root: Union[Path, str] = None):
    """Initialise toutes les ressources"""
    if app_root is None:
        app_root = Path(__file__).parent.parent
    if isinstance(app_root, str):
        app_root = Path(app_root)
    
    _intl.set_project_root(app_root.resolve())
    _config.set_project_root(app_root.resolve())
    _feature.set_project_root(app_root)
    _intl.load_all()
    _config.load_all()
    
def execute_feature(
    feature_path: str,
    module: Optional[str] = None,
    *args,
    **kwargs
) -> Any:
    """
    Exécute une fonctionnalité globale ou d'un module
    
    Args:
        feature_path: "auth.createData" (module) ou "utils.cleanData" (global)
        module: Optionnel - nom du module ("module2")
        *args: Arguments positionnels
        **kwargs: Arguments nommés
        
    Returns:
        Résultat de la fonctionnalité
    """
    return _feature.execute(feature_path, module, *args, **kwargs)

async def execute_features(
    features: List[Dict[str, Any]],
    fail_fast: bool = False
) -> Dict[str, Any]:
    """
    Exécute plusieurs fonctionnalités en parallèle
    
    Args:
        features: Liste de dictionnaires contenant:
            - feature_path: chemin de la feature
            - module: nom du module (optionnel)
            - args: arguments positionnels (optionnel)
            - kwargs: arguments nommés (optionnel)
        fail_fast: Si True, s'arrête au premier échec
    
    Returns:
        Dictionnaire avec:
            - results: résultats des features réussies
            - errors: erreurs des features échouées
    """
    return await _feature.execute_many(features, fail_fast)

def execute_features_sync(
    features: List[Dict[str, Any]],
    fail_fast: bool = False
) -> Dict[str, Any]:
    """
    Version synchrone de execute_features
    
    Args: voir execute_features
    Returns: voir execute_features
    """
    return asyncio.run(_feature.execute_many_sync(features, fail_fast))

def list_features(module: str = None) -> Dict[str, list]:
    """Liste toutes les fonctionnalités disponibles d'un module"""
    return _feature.list_features(module)

def list_all_features() -> Dict[str, Dict[str, List[str]]]:
    """Liste toutes les fonctionnalités disponibles (globales et modules)"""
    return _feature.list_all_features()

def t(key: str, locale: str = 'fr', module: str = "global", **kwargs) -> str:
    """Récupère une traduction"""
    value = _intl.get(module, key, locale) or key
    return value.format(**kwargs) if kwargs else value

def cfg(key: str, default: Any = None, module: str = "global") -> Any:
    """Récupère une configuration"""
    return _config.get(module, key, default)

def register_routes(router: APIRouter, base_path: Union[Path, str]):
    """Enregistre les routes"""
    if isinstance(base_path, str):
        base_path = Path(base_path)
    _router.register_routes(router, base_path.resolve())


async def all_requests(request: Request) -> RequestData:
    return await extract_all_request_data(request = request)

def specific_request(
    request_data: RequestData,
    data_type: str,
    key: Optional[str] = None,
    default: Any = None
) -> Any:
    return get_specific_data(
        request_data = request_data,
        data_type = data_type,
        key = key,
        default = default,
    )